package com.lzmispt.data.resource.service.metadata.impl;

package .tmp;

import com.lzmispt.core.entity.dto.listdeletemodel.ListDeleteModelDTO;
import com.lzmispt.core.entity.dto.querymodel.FilterFieldDTO;
import com.lzmispt.core.entity.dto.querymodel.FilterGroupModelDTO;
import com.lzmispt.core.entity.dto.querymodel.FilterModelDTO;
import com.lzmispt.core.entity.dto.querymodel.QueryModelDTO;
import com.lzmispt.core.service.listdelete.ListDeleteUtil;
import com.lzmispt.core.service.query.IQuery;
import com.lzmispt.data.center.entity.dto.log.CommonLogDTO;
import com.lzmispt.data.center.entity.enums.StatusCodeEnum;
import com.lzmispt.data.center.entity.vo.standard.metadata.MetadataTypeVO;
import com.lzmispt.data.resource.client.DataStandardClient;
import com.lzmispt.data.resource.dao.metadata.IMetadataDao;
import com.lzmispt.data.resource.entity.dto.metadata.MetadataElementConfigDTO;
import com.lzmispt.data.resource.entity.dto.metadata.MetadataItemDTO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataDirectoryPO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataElementConfigPO;
import com.lzmispt.data.resource.entity.vo.metadata.MetadataElementConfigVO;
import com.lzmispt.data.resource.entity.vo.metadata.MetadataItemVO;
import com.lzmispt.data.resource.service.metadata.IMetadataElement;
import com.lzmispt.dcbase.ServiceUtil;
import com.lzmispt.dcbase.database.IManage;
import com.lzmispt.dcbase.databasestruct.DataStructProvider;
import com.lzmispt.dcbase.databasestruct.TypeConverter;
import com.lzmispt.dcbase.dcconst.DCConst;
import com.lzmispt.dcbase.util.HeaderInterceptor;
import com.lzmispt.framework.entity.entity.ResultEntity;
import com.lzmispt.framework.exception.ExceptionUtil;
import com.lzmispt.framework.fconst.FrameConst;
import com.lzmispt.framework.log.LogHelper;
import com.lzmispt.framework.utils.RabbitMQUtil;
import com.lzmispt.framework.utils.WorkerId;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.StringUtils;

public abstract class MetadataElementManager implements IMetadataElement, IManage {
  private static final String DATA_TYPE = "dataType";
  
  private static final Integer ELEMENT_DATA_LENGTH = Integer.valueOf(50);
  
  private static final String LOG_TIP_MODIFY = "】修改为【";
  
  private static final String PARAM_TAG_PRE = "#{";
  
  @Autowired
  private IQuery query;
  
  @Autowired
  DataStandardClient dataStandardClient;
  
  @Value("${data-center.data-authority.manage}")
  private String manageDataAuthority;
  
  @Value("${data-center.data-mode:mysql}")
  private String dbType;
  
  public Boolean checkColumnIsExist(Long elementId, String fieldName, boolean isAdd) {
    if (isAdd)
      return Boolean.valueOf(true); 
    MetadataElementConfigPO metadataElementConfigPOFromDb = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataElementConfigById(elementId);
    if (metadataElementConfigPOFromDb != null && !metadataElementConfigPOFromDb.getElementName().equalsIgnoreCase(fieldName)) {
      MetadataElementConfigPO metadataElementConfigPO = new MetadataElementConfigPO();
      metadataElementConfigPO.setElementId(elementId);
      metadataElementConfigPO.setElementName(fieldName);
      return checkElementNameIsExist(metadataElementConfigPO, false);
    } 
    return Boolean.valueOf(true);
  }
  
  public Boolean isMetadataPublicElementHasData() {
    return Boolean.valueOf((((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).isMetadataPublicElementHasData().intValue() > 0));
  }
  
  private String getDbFieldType(MetadataElementConfigPO metadataElementConfigPO) {
    String dbFieldType = "";
    switch (metadataElementConfigPO.getElementDataType()) {
      case "String":
        dbFieldType = getDbFieldTypeMapping("String") + "(" + metadataElementConfigPO.getElementDataLength() + ")";
        break;
      case "Guid":
        dbFieldType = "varchar(" + metadataElementConfigPO.getElementDataLength() + ")";
        break;
      case "Double":
        dbFieldType = "NUMERIC(18,0)";
        break;
      case "Int":
        dbFieldType = getDbFieldTypeMapping("Int");
        break;
      case "Float":
        dbFieldType = "float";
        break;
      case "Long":
        dbFieldType = getDbFieldTypeMapping("Long");
        break;
      case "Binary":
        if (this.dbType.equalsIgnoreCase("mysql")) {
          dbFieldType = "binary(" + metadataElementConfigPO.getElementDataLength() + ")";
          break;
        } 
        dbFieldType = getDbFieldTypeMapping("Binary");
        break;
      case "DateTime":
        dbFieldType = getDbFieldTypeMapping("DateTime");
        break;
      case "Numeric":
        if (metadataElementConfigPO.getElementDataLength().intValue() == 1) {
          dbFieldType = getDbFieldTypeMapping("Numeric") + "(1,1)";
          break;
        } 
        dbFieldType = getDbFieldTypeMapping("Numeric") + "(" + metadataElementConfigPO.getElementDataLength() + ",2)";
        break;
    } 
    return dbFieldType;
  }
  
  private String getDbFieldTypeMapping(String fieldType) {
    MetadataTypeVO metadataTypeVO = (MetadataTypeVO)this.dataStandardClient.getMetadataTypeByDataTypeName(fieldType, getDefaultCode(), "tokenId").getObj();
    if (this.dbType.equalsIgnoreCase("mysql"))
      return metadataTypeVO.getMysqlDbType(); 
    if (this.dbType.equalsIgnoreCase("sqlserver"))
      return metadataTypeVO.getSqlDbType(); 
    if (this.dbType.equalsIgnoreCase("oracle"))
      return metadataTypeVO.getOracleDbType(); 
    if (this.dbType.equalsIgnoreCase("postgresql"))
      return metadataTypeVO.getPgsqlDbType(); 
    if (this.dbType.equalsIgnoreCase("dm"))
      return metadataTypeVO.getDm8DbType(); 
    if (this.dbType.equalsIgnoreCase("kingbase"))
      return metadataTypeVO.getKingBaseDbType(); 
    return metadataTypeVO.getMysqlDbType();
  }
  
  public ResultEntity<Boolean> addMetadataPublicElementColumn(MetadataElementConfigPO metadataElementConfigPO) {
    if (!metadataElementConfigPO.getElementName().matches("^[a-zA-Z0-9]+$"))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_COLUMN_NAME_NO_CONFORMITY.getCode()), StatusCodeEnum.DATA_RESOURCE_COLUMN_NAME_NO_CONFORMITY
          .getMessage(), Boolean.valueOf(false)); 
    if ("String".equals(metadataElementConfigPO.getElementDataType()) && (metadataElementConfigPO.getElementDataLength() == null || metadataElementConfigPO.getElementDataLength().intValue() == 0))
      metadataElementConfigPO.setElementDataLength(ELEMENT_DATA_LENGTH); 
    if ("Binary".equals(metadataElementConfigPO.getElementDataType()) && (metadataElementConfigPO.getElementDataLength() == null || metadataElementConfigPO.getElementDataLength().intValue() == 0))
      metadataElementConfigPO.setElementDataLength(ELEMENT_DATA_LENGTH); 
    String dataType = getDbFieldType(metadataElementConfigPO);
    try {
      ResultEntity<DataStructProvider> getDataBaseProviderRet = DataStructProvider.getDataBaseProvider(getDefaultCode());
      if (!getDataBaseProviderRet.getCode().equals(Integer.valueOf(0)))
        return ResultEntity.errorMessage(getDataBaseProviderRet.getCode(), getDataBaseProviderRet.getMsg()); 
      DataStructProvider dataStructProvider = (DataStructProvider)getDataBaseProviderRet.getObj();
      dataStructProvider.createColumn("dc_metadatapublicelement", metadataElementConfigPO.getElementName(), dataType, metadataElementConfigPO.getElementDataLength(), Integer.valueOf(0));
    } catch (Exception e) {
      LogHelper.error("addMetadataPublicElementColumn失败", ExceptionUtil.getStackTraceMsg(e));
      return ResultEntity.successMessage(Boolean.valueOf(false));
    } 
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  public ResultEntity<Boolean> updateMetadataPublicElementColumn(String oldColumnName, MetadataElementConfigPO metadataElementConfigPO) {
    if (!oldColumnName.matches("^[a-zA-Z0-9]+$"))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_COLUMN_NAME_NO_CONFORMITY.getCode()), StatusCodeEnum.DATA_RESOURCE_COLUMN_NAME_NO_CONFORMITY
          .getMessage(), Boolean.valueOf(false)); 
    if (!metadataElementConfigPO.getElementName().matches("^[a-zA-Z0-9]+$"))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_COLUMN_NAME_NO_CONFORMITY.getCode()), StatusCodeEnum.DATA_RESOURCE_COLUMN_NAME_NO_CONFORMITY
          .getMessage(), Boolean.valueOf(false)); 
    String dataType = getDbFieldType(metadataElementConfigPO);
    Map<String, String> param = new HashMap<>();
    param.put("fieldNameFromDb", oldColumnName);
    param.put("fieldName", metadataElementConfigPO.getElementName());
    param.put("dataType", dataType);
    ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).updateMetadataPublicElementColumn(param);
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  public Boolean deleteMetadataPublicElementColumn(String columnName) {
    try {
      ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).deleteMetadataPublicElementColumn(columnName);
      return Boolean.valueOf(true);
    } catch (Exception e) {
      LogHelper.error("dc_metadatapublicelement表删除列名失败", ExceptionUtil.getStackTraceMsg(e));
      return Boolean.valueOf(false);
    } 
  }
  
  public Boolean checkElementNameIsExist(MetadataElementConfigPO metadataElementConfigPO, boolean isAdd) {
    List<MetadataElementConfigPO> metadataElementConfigPOList;
    if (isAdd) {
      metadataElementConfigPOList = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataElementConfigByElementName(metadataElementConfigPO);
    } else {
      metadataElementConfigPOList = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataElementConfigByElementNameExceptId(metadataElementConfigPO);
    } 
    return Boolean.valueOf((metadataElementConfigPOList != null && metadataElementConfigPOList.size() != 0));
  }
  
  public ResultEntity<Integer> getNewElementOrder() {
    return ResultEntity.successMessage(getMaxOrder());
  }
  
  public ResultEntity<MetadataElementConfigVO> addMetadataElementConfig(MetadataElementConfigDTO metadataElementConfigDTO, String tokenId) {
    MetadataElementConfigVO metadataElementConfigVO = new MetadataElementConfigVO();
    MetadataElementConfigPO metadataElementConfigPO = new MetadataElementConfigPO();
    int newOrder = ((metadataElementConfigDTO.getElementOrder() == null) ? getMaxOrder() : metadataElementConfigDTO.getElementOrder()).intValue();
    String elementDataLength = (metadataElementConfigDTO.getElementDataLength() == null) ? "50" : metadataElementConfigDTO.getElementDataLength().toString();
    switch (metadataElementConfigDTO.getElementDataType()) {
      case "Int":
        elementDataLength = "4";
        break;
      case "Guid":
        elementDataLength = "36";
        break;
      case "Float":
      case "DateTime":
        elementDataLength = null;
        break;
      case "Numeric":
        elementDataLength = (metadataElementConfigDTO.getElementDataLength() == null) ? "19" : metadataElementConfigDTO.getElementDataLength().toString();
        break;
    } 
    metadataElementConfigDTO.setElementId((Long)WorkerId.getNextId(Long.class));
    if (elementDataLength == null) {
      metadataElementConfigDTO.setElementDataLength(Integer.valueOf(0));
    } else {
      metadataElementConfigDTO.setElementDataLength(Integer.valueOf(Integer.parseInt(elementDataLength)));
    } 
    metadataElementConfigDTO.setElementOrder(Integer.valueOf(newOrder));
    metadataElementConfigDTO.setCreateTime(new Date());
    metadataElementConfigDTO.setUpdateTime(new Date());
    metadataElementConfigDTO.setIsDelete(DCConst.FALSE);
    BeanUtils.copyProperties(metadataElementConfigDTO, metadataElementConfigVO);
    BeanUtils.copyProperties(metadataElementConfigDTO, metadataElementConfigPO);
    Boolean isExistElementName = checkElementNameIsExist(metadataElementConfigPO, true);
    if (isExistElementName.booleanValue())
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_METADATA_NAME_EXISTS.getCode()), StatusCodeEnum.DATA_RESOURCE_METADATA_NAME_EXISTS
          .getMessage()); 
    addMetadataPublicElementColumn(metadataElementConfigPO);
    if ("ORG".equals(metadataElementConfigPO.getElementDataType()))
      metadataElementConfigPO.setElementDataLength(null); 
    ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).addMetadataElementConfig(metadataElementConfigPO);
    String name = StringUtils.isEmpty(metadataElementConfigDTO.getElementCnName()) ? metadataElementConfigDTO.getElementName() : metadataElementConfigDTO.getElementCnName();
    writeSystemLog("新增元数据 新增元数据【" + name + "】", tokenId);
    return ResultEntity.successMessage(metadataElementConfigVO);
  }
  
  public ResultEntity<MetadataElementConfigVO> updateMetadataElementConfig(MetadataElementConfigDTO metadataElementConfigDTO, String tokenId) {
    MetadataElementConfigVO metadataElementConfigVO = new MetadataElementConfigVO();
    MetadataElementConfigPO metadataElementConfigPO = new MetadataElementConfigPO();
    int newOrder = ((metadataElementConfigDTO.getElementOrder() == null) ? getMaxOrder() : metadataElementConfigDTO.getElementOrder()).intValue();
    metadataElementConfigDTO.setElementOrder(Integer.valueOf(newOrder));
    String elementDataLength = (metadataElementConfigDTO.getElementDataLength() == null) ? "" : metadataElementConfigDTO.getElementDataLength().toString();
    MetadataElementConfigPO metadataElementConfigPOFromDb = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataElementConfigById(metadataElementConfigDTO.getElementId());
    if (metadataElementConfigPOFromDb == null)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CORE_METADATA_NO_EXISTS.getCode()), StatusCodeEnum.DATA_RESOURCE_CORE_METADATA_NO_EXISTS
          .getMessage()); 
    String elementNameFromDb = metadataElementConfigPOFromDb.getElementName();
    Integer elementDataLengthFromDb = metadataElementConfigPOFromDb.getElementDataLength();
    metadataElementConfigDTO.setUpdateTime(new Date());
    BeanUtils.copyProperties(metadataElementConfigDTO, metadataElementConfigVO);
    BeanUtils.copyProperties(metadataElementConfigDTO, metadataElementConfigPO);
    if (!elementNameFromDb.equalsIgnoreCase(metadataElementConfigDTO.getElementName())) {
      BeanUtils.copyProperties(metadataElementConfigDTO, metadataElementConfigPO);
      Boolean isExistElementName = checkElementNameIsExist(metadataElementConfigPO, false);
      if (isExistElementName.booleanValue())
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_METADATA_NAME_EXISTS.getCode()), StatusCodeEnum.DATA_RESOURCE_METADATA_NAME_EXISTS
            .getMessage()); 
      updateMetadataPublicElementColumn(elementNameFromDb, metadataElementConfigPO);
    } 
    ResultEntity<DataStructProvider> getDataBaseProviderRet = DataStructProvider.getDataBaseProvider(getDefaultCode());
    if (!getDataBaseProviderRet.getCode().equals(Integer.valueOf(0)))
      return ResultEntity.errorMessage(getDataBaseProviderRet.getCode(), getDataBaseProviderRet.getMsg()); 
    DataStructProvider dataStructProvider = (DataStructProvider)getDataBaseProviderRet.getObj();
    try {
      MetadataTypeVO metadataTypeVO = (MetadataTypeVO)this.dataStandardClient.getMetadataTypeByDataTypeName(metadataElementConfigPO.getElementDataType(), getDefaultCode(), tokenId).getObj();
      if (metadataTypeVO != null) {
        String dbFieldType = getDbFieldType(metadataElementConfigPO);
        boolean alterSuccess = dataStructProvider.alterColumn("dc_metadatapublicelement", metadataElementConfigPO.getElementDataLength(), 0, dbFieldType, true, metadataElementConfigPO.getElementName(), false, false);
        if (!alterSuccess) {
          if (!elementNameFromDb.equalsIgnoreCase(metadataElementConfigDTO.getElementName())) {
            String nowName = metadataElementConfigPO.getElementName();
            metadataElementConfigPO.setElementName(elementNameFromDb);
            updateMetadataPublicElementColumn(nowName, metadataElementConfigPO);
          } 
          return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_UPDATE_METADATA_ELEMENT_CONFIG_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_UPDATE_METADATA_ELEMENT_CONFIG_FAILED.getMessage());
        } 
      } 
    } catch (Exception e) {
      LogHelper.error("更新字段信息失败", ExceptionUtil.getStackTraceMsg(e));
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_UPDATE_METADATA_ELEMENT_CONFIG_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_UPDATE_METADATA_ELEMENT_CONFIG_FAILED.getMessage());
    } 
    if (!StringUtils.isEmpty(elementDataLength))
      metadataElementConfigDTO.setElementDataLength(Integer.valueOf(Integer.parseInt(elementDataLength))); 
    ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).updateMetadataElementConfig(metadataElementConfigPO);
    updateMetadataElementConfigLog(metadataElementConfigDTO, metadataElementConfigPOFromDb, tokenId);
    return ResultEntity.successMessage(metadataElementConfigVO);
  }
  
  public Boolean deleteMetadataElementConfigById(Long elementId) {
    return Boolean.valueOf((((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).deleteMetadataElementConfigById(elementId) > 0));
  }
  
  public ResultEntity<Boolean> deleteMetadataElementConfigByIds(ListDeleteModelDTO listDeleteModelDTO, String tokenId) {
    ResultEntity<String> result = ListDeleteUtil.getListDeleteWhere(listDeleteModelDTO, this.dbType);
    if (result.getCode().intValue() != 0)
      return ResultEntity.errorMessage(result.getCode(), result.getMsg()); 
    String sql = StringUtils.isEmpty(result.getObj()) ? "isdelete =0" : ((String)result.getObj() + "and isdelete =0");
    List<MetadataElementConfigPO> metadataElementConfigList = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataElementConfig(sql);
    StringBuilder logInfo = new StringBuilder();
    for (MetadataElementConfigPO metadataElementConfigPO : metadataElementConfigList) {
      String name = StringUtils.isEmpty(metadataElementConfigPO.getElementCnName()) ? metadataElementConfigPO.getElementName() : metadataElementConfigPO.getElementCnName();
      if (logInfo.length() == 0) {
        logInfo.append("删除元数据 删除元数据【").append(name).append("】");
      } else {
        logInfo.append("、【").append(name).append("】");
      } 
      deleteMetadataPublicElementColumn(metadataElementConfigPO.getElementName());
      deleteMetadataElementConfigById(metadataElementConfigPO.getElementId());
    } 
    if (logInfo.length() != 0)
      writeSystemLog(logInfo.toString(), tokenId); 
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  public MetadataElementConfigVO getMetadataElementConfigById(Long elementId) {
    MetadataElementConfigVO metadataElementConfigVO = new MetadataElementConfigVO();
    MetadataElementConfigPO metadataElementConfigPO = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataElementConfigById(elementId);
    if (metadataElementConfigPO != null)
      BeanUtils.copyProperties(metadataElementConfigPO, metadataElementConfigVO); 
    return metadataElementConfigVO;
  }
  
  public ResultEntity<List<MetadataElementConfigVO>> getMetadataElementConfigList(QueryModelDTO queryModelDTO, Long userId, Long organizationId) {
    List<MetadataElementConfigPO> metadataElementConfigPOList;
    FilterFieldDTO user, org, org1;
    List<MetadataElementConfigVO> metadataElementConfigVOList = new ArrayList<>();
    if (queryModelDTO == null)
      return ResultEntity.successMessage(metadataElementConfigVOList); 
    if (queryModelDTO.getFilterModel() == null) {
      FilterModelDTO filterModelDTO = new FilterModelDTO();
      filterModelDTO.setLogicType(Integer.valueOf(0));
      queryModelDTO.setFilterModel(filterModelDTO);
      List<FilterGroupModelDTO> list = new ArrayList<>();
      filterModelDTO.setFilterGroupModel(list);
    } 
    List<FilterGroupModelDTO> filterGroupModelDTOList = new ArrayList<>();
    FilterGroupModelDTO filterGroupModelDTO = new FilterGroupModelDTO();
    filterGroupModelDTO.setLogicType(Integer.valueOf(0));
    List<FilterFieldDTO> filterFieldDTOList = new ArrayList<>();
    FilterFieldDTO filterFieldDTO = new FilterFieldDTO();
    filterFieldDTO.setFieldName("isdelete");
    filterFieldDTO.setOperationType(Integer.valueOf(20));
    filterFieldDTO.setFieldType(Integer.valueOf(30));
    filterFieldDTO.setFieldValue(Integer.valueOf(0));
    filterFieldDTOList.add(filterFieldDTO);
    switch (this.manageDataAuthority) {
      case "userorg":
        user = new FilterFieldDTO();
        user.setFieldName("userid");
        user.setOperationType(Integer.valueOf(20));
        user.setFieldType(Integer.valueOf(40));
        user.setFieldValue(userId);
        org = new FilterFieldDTO();
        org.setFieldName("organizationid");
        org.setOperationType(Integer.valueOf(20));
        org.setFieldType(Integer.valueOf(40));
        org.setFieldValue(organizationId);
        filterFieldDTOList.add(user);
        filterFieldDTOList.add(org);
        break;
      case "org":
        org1 = new FilterFieldDTO();
        org1.setFieldName("organizationid");
        org1.setOperationType(Integer.valueOf(20));
        org1.setFieldType(Integer.valueOf(40));
        org1.setFieldValue(organizationId);
        filterFieldDTOList.add(org1);
        break;
    } 
    filterGroupModelDTO.setFilterFields(filterFieldDTOList);
    filterGroupModelDTOList.add(filterGroupModelDTO);
    queryModelDTO.getFilterModel().getFilterGroupModel().addAll(filterGroupModelDTOList);
    boolean isEscapeStr = false;
    if (this.dbType.equalsIgnoreCase("sqlserver"))
      isEscapeStr = true; 
    ResultEntity<String> sql = this.query.getWhereSqlStr(queryModelDTO, this.dbType, true, isEscapeStr);
    if (sql.getCode().intValue() == 0) {
      metadataElementConfigPOList = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataElementConfig((String)sql.getObj());
    } else {
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_STANDARD_GET_WHERE_QUERY_FAIL.getCode()), StatusCodeEnum.DATA_STANDARD_GET_WHERE_QUERY_FAIL
          .getMessage());
    } 
    if (metadataElementConfigPOList != null && metadataElementConfigPOList.size() > 0)
      for (MetadataElementConfigPO metadataElementConfigPO : metadataElementConfigPOList) {
        MetadataElementConfigVO metadataElementConfigVO = new MetadataElementConfigVO();
        BeanUtils.copyProperties(metadataElementConfigPO, metadataElementConfigVO);
        metadataElementConfigVOList.add(metadataElementConfigVO);
      }  
    return ResultEntity.successMessage(metadataElementConfigVOList);
  }
  
  public String validateValueIsFitFieldType(MetadataItemDTO metadataItemDTO) {
    Pattern pattern;
    String checkFailPrompt = "";
    String elementValue = metadataItemDTO.getElementValue() + "";
    String metadataTipStart = "元数据【";
    switch (metadataItemDTO.getElementDataType()) {
      case "Guid":
        try {
          UUID uUID = UUID.fromString(elementValue);
        } catch (Exception e) {
          checkFailPrompt = checkFailPrompt + metadataTipStart + metadataItemDTO.getElementCnName() + "】的数据类型为:" + metadataItemDTO.getElementDataType() + ",输入的值与其数据类型不符";
        } 
        break;
      case "DateTime":
        try {
          SimpleDateFormat formatDateTime = new SimpleDateFormat("yyyy-MM-dd");
          formatDateTime.parse(elementValue);
        } catch (Exception e) {
          checkFailPrompt = checkFailPrompt + metadataTipStart + metadataItemDTO.getElementCnName() + "】的数据类型为:日期,输入的值与其数据类型不符";
        } 
        break;
      case "Float":
        try {
          Float.parseFloat(elementValue);
        } catch (Exception e) {
          checkFailPrompt = checkFailPrompt + metadataTipStart + metadataItemDTO.getElementCnName() + "】的数据类型为:浮点数,您所输入的值与其数据类型不符！";
        } 
        break;
      case "Int":
        pattern = Pattern.compile("^-?\\d+$");
        if (!pattern.matcher(elementValue).matches())
          checkFailPrompt = checkFailPrompt + metadataTipStart + metadataItemDTO.getElementCnName() + "】的数据类型为:整数,输入的值与其数据类型不符！"; 
        break;
      case "String":
        try {
          if (elementValue.length() > metadataItemDTO.getElementDataLength().intValue())
            checkFailPrompt = checkFailPrompt + "输入的内容的长度超过了元数据【" + metadataItemDTO.getElementCnName() + "】所允许的长度！"; 
        } catch (Exception e) {
          checkFailPrompt = checkFailPrompt + metadataTipStart + metadataItemDTO.getElementCnName() + "】的数据类型为:字符串,输入的值与其数据类型不符！";
        } 
        break;
      case "Long":
        if (elementValue.length() > metadataItemDTO.getElementDataLength().intValue()) {
          checkFailPrompt = checkFailPrompt + "输入的内容的长度超过了元数据【" + metadataItemDTO.getElementCnName() + "】所允许的长度！";
          break;
        } 
        try {
          Long.valueOf(elementValue);
        } catch (Exception e) {
          checkFailPrompt = checkFailPrompt + metadataTipStart + metadataItemDTO.getElementCnName() + "】的数据类型为:长整数,输入的值与其数据类型不符！";
        } 
        break;
      case "Double":
        try {
          Double.parseDouble(elementValue);
        } catch (Exception e) {
          checkFailPrompt = checkFailPrompt + metadataTipStart + metadataItemDTO.getElementCnName() + "】的数据类型为:双精度类型,输入的值与其数据类型不符，请重新输入！";
        } 
        break;
      case "Numeric":
        try {
          new BigDecimal(elementValue);
        } catch (Exception e) {
          checkFailPrompt = checkFailPrompt + metadataTipStart + metadataItemDTO.getElementCnName() + "】的数据类型为:数值类型,输入的值与其数据类型不符！";
        } 
        break;
    } 
    return checkFailPrompt;
  }
  
  public ResultEntity<Boolean> saveMetadataPublicElement(List<MetadataItemDTO> metadataItemDTOList, String tokenId) {
    if (metadataItemDTOList == null || metadataItemDTOList.size() == 0)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SAVE_METADATA_PUBLIC_ELEMENT_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SAVE_METADATA_PUBLIC_ELEMENT_FAILED
          .getMessage(), Boolean.valueOf(false)); 
    int requiredFieldCount = 0;
    int realityRequiredFieldCount = 0;
    StringBuilder requiredFieldPrompt = new StringBuilder();
    StringBuilder checkFailPrompt = new StringBuilder();
    for (MetadataItemDTO metadataItemDTO : metadataItemDTOList) {
      if (metadataItemDTO.getIsRequired().intValue() == 1)
        requiredFieldCount++; 
    } 
    boolean validateRequiredFlag = true;
    for (MetadataItemDTO metadataItemDTO : metadataItemDTOList) {
      if (metadataItemDTO.getIsRequired().intValue() == 1) {
        if (metadataItemDTO.getElementValue() != null && !StringUtils.isEmpty(metadataItemDTO.getElementValue().toString())) {
          checkFailPrompt.append(validateValueIsFitFieldType(metadataItemDTO));
          realityRequiredFieldCount++;
          if (requiredFieldCount == realityRequiredFieldCount)
            validateRequiredFlag = true; 
          continue;
        } 
        requiredFieldPrompt.append("【").append(metadataItemDTO.getElementCnName()).append("】");
        validateRequiredFlag = false;
        continue;
      } 
      if (!StringUtils.isEmpty(metadataItemDTO.getElementValue() + "")) {
        checkFailPrompt.append(validateValueIsFitFieldType(metadataItemDTO));
        continue;
      } 
      metadataItemDTO.setElementValue(null);
    } 
    if (!validateRequiredFlag)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_MANDATORY_FIELD_NO_EMPTY.getCode()), requiredFieldPrompt + "是必填字段，不能为空", Boolean.valueOf(false)); 
    if (checkFailPrompt.length() != 0)
      return ResultEntity.errorMessage(Integer.valueOf(-1), checkFailPrompt.toString(), Boolean.valueOf(false)); 
    boolean res = updateMetadataItems(metadataItemDTOList, tokenId).booleanValue();
    return ResultEntity.successMessage(Boolean.valueOf(res));
  }
  
  public Boolean updateMetadataItems(List<MetadataItemDTO> metadataItemDTOList, String tokenId) {
    List<String> metadataPublicElementColumnNames = getMetadataPublicElementColumnNames();
    if (metadataPublicElementColumnNames == null || metadataPublicElementColumnNames.size() == 0)
      return Boolean.valueOf(false); 
    List<MetadataItemDTO> metadataItemList = new ArrayList<>();
    for (MetadataItemDTO metadataItemDTO : metadataItemDTOList) {
      if (this.dbType.equalsIgnoreCase("kingbase"))
        metadataItemDTO.setElementName(handKingBaseFieldName(metadataItemDTO.getElementName())); 
      if (metadataPublicElementColumnNames.contains(metadataItemDTO.getElementName())) {
        if ("Binary".equalsIgnoreCase(metadataItemDTO.getElementDataType()))
          metadataItemDTO.setElementValue(!StringUtils.isEmpty(metadataItemDTO.getElementValue()) ? String.valueOf(metadataItemDTO.getElementValue()).getBytes(StandardCharsets.UTF_8) : null); 
        metadataItemList.add(metadataItemDTO);
      } 
    } 
    addSysField(metadataItemList);
    if (metadataItemList.size() > 0) {
      Map metadataPublicElement = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataPublicElementById(((MetadataItemDTO)metadataItemList.get(0)).getDirectoryId());
      if (metadataPublicElement == null || metadataPublicElement.size() == 0) {
        Map<String, Object> fieldList = new HashMap<>();
        fieldList.put("directoryid", ((MetadataItemDTO)metadataItemList.get(0)).getDirectoryId());
        fieldList.put("entityid", ((MetadataItemDTO)metadataItemList.get(0)).getEntityId());
        for (MetadataItemDTO metadataItemDTO : metadataItemList)
          fieldList.put(metadataItemDTO.getElementName(), metadataItemDTO.getElementValue()); 
        DataStructProvider dataStructProvider = (DataStructProvider)DataStructProvider.getDataBaseProvider(getDefaultCode()).getObj();
        try {
          String insertSql = buildInsertSql("dc_metadatapublicelement", metadataItemList);
          dataStructProvider.execCustomSql(insertSql, fieldList);
        } catch (Exception e) {
          LogHelper.error("保存核心数据元记录日志失败", ExceptionUtil.getStackTraceMsg(e));
        } 
      } else {
        Map<String, Object> param = new HashMap<>();
        param.put("directoryId", ((MetadataItemDTO)metadataItemList.get(0)).getDirectoryId());
        Map<String, Object> fieldList = new HashMap<>();
        for (MetadataItemDTO metadataItemDTO : metadataItemList)
          fieldList.put(metadataItemDTO.getElementName(), metadataItemDTO.getElementValue()); 
        param.put("fieldList", fieldList);
        ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).dynamicUpdateMetadataPublicElement(param);
      } 
      try {
        recordSaveLog(metadataPublicElement, metadataItemList, tokenId);
      } catch (Exception e) {
        LogHelper.error("保存核心数据元记录日志失败", ExceptionUtil.getStackTraceMsg(e));
      } 
    } 
    return Boolean.valueOf(true);
  }
  
  private void addSysField(List<MetadataItemDTO> metadataItemList) {
    MetadataItemDTO directoryIdItem = new MetadataItemDTO();
    directoryIdItem.setDirectoryId(((MetadataItemDTO)metadataItemList.get(0)).getDirectoryId());
    directoryIdItem.setEntityId(((MetadataItemDTO)metadataItemList.get(0)).getEntityId());
    directoryIdItem.setElementName("directoryid");
    directoryIdItem.setElementCnName("directoryid");
    directoryIdItem.setElementValue(((MetadataItemDTO)metadataItemList.get(0)).getDirectoryId());
    directoryIdItem.setElementDataType("Long");
    metadataItemList.add(directoryIdItem);
    MetadataItemDTO entityIdItem = new MetadataItemDTO();
    entityIdItem.setDirectoryId(((MetadataItemDTO)metadataItemList.get(0)).getEntityId());
    entityIdItem.setEntityId(((MetadataItemDTO)metadataItemList.get(0)).getEntityId());
    entityIdItem.setElementName("entityid");
    entityIdItem.setElementCnName("entityid");
    entityIdItem.setElementValue(((MetadataItemDTO)metadataItemList.get(0)).getEntityId());
    entityIdItem.setElementDataType("Long");
    metadataItemList.add(entityIdItem);
  }
  
  private String buildInsertSql(String tableName, List<MetadataItemDTO> metadataItemList) {
    StringBuilder insertColumnNameSql = new StringBuilder();
    StringBuilder insertColumnValueSql = new StringBuilder();
    for (MetadataItemDTO metadataItemDTO : metadataItemList) {
      if (insertColumnNameSql.length() == 0) {
        insertColumnNameSql.append(TypeConverter.getColumnSqlStr(metadataItemDTO.getElementName(), this.dbType));
      } else {
        insertColumnNameSql.append(",").append(TypeConverter.getColumnSqlStr(metadataItemDTO.getElementName(), this.dbType));
      } 
      if (insertColumnValueSql.length() == 0) {
        insertColumnValueSql.append("#{").append(metadataItemDTO.getElementName()).append("}");
        continue;
      } 
      insertColumnValueSql.append(",").append("#{").append(metadataItemDTO.getElementName()).append("}");
    } 
    StringBuilder sql = new StringBuilder();
    sql.append("insert into ").append(tableName).append(" (").append(insertColumnNameSql).append(") ").append(" values (").append(insertColumnValueSql).append(")");
    return sql.toString();
  }
  
  public boolean isAcronym(String word) {
    for (int i = 0; i < word.length(); i++) {
      char c = word.charAt(i);
      if (Character.isLowerCase(c))
        return false; 
    } 
    return true;
  }
  
  private String handKingBaseFieldName(String fieldName) {
    if (this.dbType.equalsIgnoreCase("kingbase") && fieldName != null && isAcronym(fieldName))
      fieldName = fieldName.toLowerCase(); 
    return fieldName;
  }
  
  public Boolean deleteMetadataPublicElementById(Long directoryId) {
    return Boolean.valueOf((((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).deleteMetadataPublicElementById(directoryId).intValue() > 0));
  }
  
  public Boolean deleteMetadataPublicElementByIds(List<Long> directoryIds) {
    if (directoryIds == null || directoryIds.size() == 0)
      return Boolean.valueOf(false); 
    for (Long directoryId : directoryIds)
      deleteMetadataPublicElementById(directoryId); 
    return Boolean.valueOf(true);
  }
  
  public Map getMetadataPublicElementById(Long directoryId) {
    return ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataPublicElementById(directoryId);
  }
  
  public List<String> getMetadataPublicElementColumnNames() {
    return ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataPublicElementColumnNames();
  }
  
  public ResultEntity<List<MetadataItemVO>> getMetadataItemList(Long directoryId) {
    List<MetadataItemVO> metadataItemList = new ArrayList<>();
    List<MetadataElementConfigPO> metadataElementConfigPOList = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataItemConfig();
    if (metadataElementConfigPOList == null || metadataElementConfigPOList.size() == 0)
      return ResultEntity.successMessage(metadataItemList); 
    for (MetadataElementConfigPO metadataElementConfig : metadataElementConfigPOList) {
      MetadataItemVO metadataItemVO = new MetadataItemVO();
      metadataElementConfig.setElementName(handKingBaseFieldName(metadataElementConfig.getElementName()));
      BeanUtils.copyProperties(metadataElementConfig, metadataItemVO);
      metadataItemList.add(metadataItemVO);
    } 
    Map map = getMetadataPublicElementById(directoryId);
    if (map != null && !map.isEmpty())
      for (MetadataItemVO metadataItem : metadataItemList) {
        if (map.containsKey(metadataItem.getElementName())) {
          if ("Binary".equalsIgnoreCase(metadataItem.getElementDataType())) {
            metadataItem.setElementValue((new String((byte[])map.get(metadataItem.getElementName()))).trim());
            continue;
          } 
          metadataItem.setElementValue((new StringBuilder()).append(map.get(metadataItem.getElementName())).append("").toString());
        } 
      }  
    return ResultEntity.successMessage(metadataItemList);
  }
  
  private void updateMetadataElementConfigLog(MetadataElementConfigDTO metadataElementConfigDTO, MetadataElementConfigPO metadataElementConfigPOFromDb, String tokenId) {
    String elementNameFromDb = metadataElementConfigPOFromDb.getElementName();
    String elementCnNameFromDb = metadataElementConfigPOFromDb.getElementCnName();
    String elementDataTypeFromDb = metadataElementConfigPOFromDb.getElementDataType();
    Integer elementDataLengthFromDb = metadataElementConfigPOFromDb.getElementDataLength();
    Integer isSystemFromDb = metadataElementConfigPOFromDb.getIsSystem();
    String elementDataLength = (metadataElementConfigDTO.getElementDataLength() == null) ? "" : metadataElementConfigDTO.getElementDataLength().toString();
    StringBuilder logInfo = new StringBuilder();
    String logStart = "修改元数据 元数据【";
    String name = StringUtils.isEmpty(metadataElementConfigDTO.getElementCnName()) ? metadataElementConfigDTO.getElementName() : metadataElementConfigDTO.getElementCnName();
    if (!elementNameFromDb.equalsIgnoreCase(metadataElementConfigDTO.getElementName()))
      logInfo.append("修改元数据 名称从【").append(elementNameFromDb).append("】修改为【").append(metadataElementConfigDTO.getElementName()).append("】"); 
    if (!elementCnNameFromDb.equalsIgnoreCase(metadataElementConfigDTO.getElementCnName()))
      if (logInfo.length() == 0) {
        logInfo.append("修改元数据 中文名从【").append(elementCnNameFromDb).append("】修改为【").append(metadataElementConfigDTO.getElementCnName()).append("】");
      } else {
        logInfo.append(",中文名从【").append(elementCnNameFromDb).append("】修改为【").append(metadataElementConfigDTO.getElementCnName()).append("】");
      }  
    if (!elementDataTypeFromDb.equalsIgnoreCase(metadataElementConfigDTO.getElementDataType()))
      if (logInfo.length() == 0) {
        logInfo.append(logStart).append(name).append("】的数据类型从【").append(elementDataTypeFromDb).append("】修改为【").append(metadataElementConfigDTO.getElementDataType()).append("】");
      } else {
        logInfo.append(",元数据【").append(name).append("】的数据类型从【").append(elementDataTypeFromDb).append("】修改为【").append(metadataElementConfigDTO.getElementDataType()).append("】");
      }  
    if (!StringUtils.isEmpty(elementDataLength) && 
      elementDataLengthFromDb.intValue() != Integer.parseInt(elementDataLength))
      if (logInfo.length() == 0) {
        logInfo.append(logStart).append(name).append("】的长度从【").append(elementDataLengthFromDb).append("】修改为【").append(elementDataLength).append("】");
      } else {
        logInfo.append(",元数据【").append(name).append("】的长度从【").append(elementDataLengthFromDb).append("】修改为【").append(elementDataLength).append("】");
      }  
    if (metadataElementConfigDTO.getIsSystem() != null && !isSystemFromDb.equals(metadataElementConfigDTO.getIsSystem()))
      if (logInfo.length() == 0) {
        if (metadataElementConfigDTO.getIsSystem().intValue() == 1) {
          logInfo.append(logStart).append(name).append("】设置为系统内置属性");
        } else {
          logInfo.append(logStart).append(name).append("】取消系统内置属性设置");
        } 
      } else if (metadataElementConfigDTO.getIsSystem().intValue() == 1) {
        logInfo.append(",元数据【").append(name).append("】设置为系统内置属性");
      } else {
        logInfo.append(",元数据【").append(name).append("】取消系统内置属性设置");
      }  
    if (logInfo.length() > 0)
      writeSystemLog(logInfo.toString(), tokenId); 
  }
  
  private Integer getMaxOrder() {
    Integer order = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataElementMaxOrderFromDb();
    return Integer.valueOf((order == null) ? 1 : (
        
        order.equals(Integer.valueOf(2147483647)) ? Integer.MAX_VALUE : (order
        
        .intValue() + 1)));
  }
  
  private void recordSaveLog(Map oldData, List<MetadataItemDTO> metadataItemList, String tokenId) {
    String name;
    boolean needCompare = (oldData != null);
    List<MetadataElementConfigPO> metadataItemConfig = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getMetadataItemConfig();
    MetadataDirectoryPO directoryPO = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).selectDirectoryByDirectoryId(((MetadataItemDTO)metadataItemList.get(0)).getDirectoryId());
    if (directoryPO == null)
      return; 
    switch (directoryPO.getDirectoryType().intValue()) {
      case 0:
        name = "目录";
        break;
      case 1:
        name = "数据资源";
        break;
      case 2:
        name = "视图";
        break;
      default:
        name = "";
        break;
    } 
    StringBuilder logInfo = new StringBuilder();
    String startMsg = "保存元数据配置 " + name + "【" + directoryPO.getDirectoryName() + "】保存元数据配置 ";
    Map<String, String> elementName = (Map<String, String>)metadataItemConfig.stream().collect(Collectors.toMap(MetadataElementConfigPO::getElementName, MetadataElementConfigPO::getElementCnName));
    for (MetadataItemDTO metadataItemDTO : metadataItemList) {
      String newElement = getStringValue(metadataItemDTO.getElementValue(), metadataItemDTO.getElementDataType());
      String oldElement = "";
      if (needCompare) {
        oldElement = getStringValue(oldData.get(metadataItemDTO.getElementName()), metadataItemDTO.getElementDataType());
        if (oldElement.equals(newElement))
          continue; 
      } else if (newElement.isEmpty()) {
        continue;
      } 
      logInfo.append(elementName.get(metadataItemDTO.getElementName())).append("从").append("【").append(oldElement).append("】修改为【").append(newElement).append("】,");
    } 
    if (logInfo.length() != 0)
      writeSystemLog(startMsg + logInfo.substring(0, logInfo.length() - 1), tokenId); 
  }
  
  private void writeSystemLog(String msg, String tokenId) {
    CommonLogDTO logDTO = new CommonLogDTO();
    logDTO.setPlatCode(getDefaultCode());
    logDTO.setTokenId(tokenId);
    logDTO.setLoggerInformation(msg);
    logDTO.setLogLevel(FrameConst.Log.LOG_LEVEL_SYSTEM);
    logDTO.setLogType(0);
    logDTO.setModule_code(HeaderInterceptor.getModuleCode());
    RabbitMQUtil.publish("DC.Standard.log", logDTO);
  }
  
  private String getStringValue(Object value, String type) {
    DecimalFormat df;
    SimpleDateFormat sdf;
    if (value == null)
      return ""; 
    switch (type) {
      case "String":
      case "Guid":
        return value.toString();
      case "Float":
      case "Double":
      case "Numeric":
        df = new DecimalFormat("#.00");
        return df.format(Double.valueOf(value.toString()));
      case "Int":
      case "Long":
        return String.valueOf(value);
      case "Binary":
        if (value instanceof String)
          return value.toString(); 
        return (new String((byte[])value)).trim();
      case "DateTime":
        if (value instanceof String)
          return value.toString(); 
        sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return sdf.format(value);
    } 
    return "";
  }
}
